@@ -6,7 +6,7 @@ import json
 import logging
 from collections import UserList
 from functools import partial, wraps
-from collections.abc import Mapping
+
 from .session import chose_impl, next_client_event, get_current_task_id, get_current_session
 from .utils import random_str
 
@@ -75,7 +75,7 @@ class Output:
         # For Context manager
         self.enabled_context_manager = False
         self.container_selector = None
-        self.container_dom_id = None
+        self.container_dom_id = None  # todo: this name is ambiguous, rename it to `scope_name` or others
         self.custom_enter = None
         self.custom_exit = None
 
@@ -128,6 +128,8 @@ class Output:
     def style(self, css_style):
         """Set css style for output
 
+        :param str css_style: CSS style string
+
         Example::
 
             put_text('hello').style('color: red; font-size: 20px')
@@ -142,6 +144,16 @@ class Output:
         self.spec['style'] += ';%s' % css_style
         return self
 
+    def onclick(self, callback):
+        """Add click callback to this widget.
+
+        :param callable callback: Callback which will be called when the widget is clicked.
+        """
+        callback_id = output_register_callback(lambda _: callback())
+        self.spec.setdefault('click_callback_id', '')
+        self.spec['click_callback_id'] += callback_id
+        return self
+
     def __del__(self):
         """返回值没有被变量接收时的操作：直接输出消息"""
         if not self.processed:
@@ -337,8 +349,10 @@ def input_event_handle(item_valid_funcs, form_valid_funcs, preprocess_funcs, onc
                         try:
                             onblur_name, error_msg = v_res
                         except Exception:
+                            # Use `raise Exception from None` to disable exception chaining
+                            # see: https://docs.python.org/3/tutorial/errors.html#exception-chaining
                             raise ValueError("The `validate` function for input group must "
-                                             "return `(name, error_msg)` when validation failed.")
+                                             "return `(name, error_msg)` when validation failed.") from None
 
                         send_msg('update_input', dict(target_name=onblur_name, attributes={
                             'valid_status': False,
