@@ -16,9 +16,10 @@ import tornado.ioloop
 from tornado.web import StaticFileHandler
 from tornado.websocket import WebSocketHandler
 
-from . import utils
+from . import page
 from .remote_access import start_remote_access_service
-from .utils import make_applications, render_page, cdn_validation, deserialize_binary_event
+from .page import make_applications, render_page
+from .utils import cdn_validation, deserialize_binary_event, print_listen_address
 from ..session import CoroutineBasedSession, ThreadBasedSession, ScriptModeSession, \
     register_session_implement_for_target, Session
 from ..session.base import get_session_info_from_headers
@@ -49,8 +50,8 @@ def _check_origin(origin, allowed_origins, handler: WebSocketHandler):
         return True
 
     return any(
-        fnmatch.fnmatch(origin, patten)
-        for patten in allowed_origins
+        fnmatch.fnmatch(origin, pattern)
+        for pattern in allowed_origins
     )
 
 
@@ -293,38 +294,26 @@ def start_server(applications, port=0, host='',
 
     :param list/dict/callable applications: PyWebIO application.
        Can be a task function, a list of functions, or a dictionary.
-
-       When it is a dictionary, whose key is task name and value is task function.
-       When it is a list, using function name as task name.
-
-       You can select the task to run through the ``app`` URL parameter (for example, visit ``http://host:port/?app=foo`` to run the ``foo`` task),
-       By default, the ``index`` task function is used. When the ``index`` task does not exist, PyWebIO will provide a default index home page.
-       See also :ref:`Server mode <server_and_script_mode>`
+       Refer to :ref:`Advanced topic: Multiple applications in start_server() <multiple_app>` for more information.
 
        When the task function is a coroutine function, use :ref:`Coroutine-based session <coroutine_based_session>` implementation,
        otherwise, use thread-based session implementation.
     :param int port: The port the server listens on.
        When set to ``0``, the server will automatically select a available port.
-    :param str host: The host the server listens on. ``host`` may be either an IP address or hostname. If it’s a hostname, the server will listen on all IP addresses associated with the name. ``host`` may be an empty string or None to listen on all available interfaces.
+    :param str host: The host the server listens on. ``host`` may be either an IP address or hostname.
+       If it’s a hostname, the server will listen on all IP addresses associated with the name.
+       ``host`` may be an empty string or None to listen on all available interfaces.
     :param bool debug: Tornado Server's debug mode. If enabled, the server will automatically reload for code changes.
        See `tornado doc <https://www.tornadoweb.org/en/stable/guide/running.html#debug-mode>`_ for more detail.
     :param bool/str cdn: Whether to load front-end static resources from CDN, the default is ``True``.
        Can also use a string to directly set the url of PyWebIO static resources.
     :param str static_dir: The directory to store the application static files.
        The files in this directory can be accessed via ``http://<host>:<port>/static/files``.
-       For example, if there is a ``A/B.jpg`` file in ``http_static_dir`` path,
+       For example, if there is a ``A/B.jpg`` file in ``static_dir`` path,
        it can be accessed via ``http://<host>:<port>/static/A/B.jpg``.
-    :param bool/dict remote_access: Whether to enable remote access, when enabled,
+    :param bool remote_access: Whether to enable remote access, when enabled,
        you can get a temporary public network access address for the current application,
        others can access your application via this address.
-       Using remote access makes it easy to temporarily share the application with others.
-       The remote access service is provided by `localhost.run <https://localhost.run/>`_.
-       You can use a dict to config remote access service, the following configurations are currently supported:
-
-       - ``ssh_key_path``: Use a custom ssh key, the default key path is ``~/.ssh/id_xxx``. Note that only rsa and ed25519 keys are supported.
-       - ``custom_domain``: Use a custom domain for your remote access address. This need a subscription to localhost.run.
-         See also: `Custom Domains - localhost.run <https://localhost.run/docs/custom-domains/>`_
-
     :param bool auto_open_webbrowser: Whether or not auto open web browser when server is started (if the operating system allows it) .
     :param int reconnect_timeout: The client can reconnect to server within ``reconnect_timeout`` seconds after an unexpected disconnection.
        If set to 0 (default), once the client disconnects, the server session will be closed.
@@ -341,7 +330,8 @@ def start_server(applications, port=0, host='',
 
         For detail, see `Python Doc <https://docs.python.org/zh-tw/3/library/fnmatch.html>`_
     :param callable check_origin: The validation function for request source.
-       It receives the source string (which contains protocol, host, and port parts) as parameter and return ``True/False`` to indicate that the server accepts/rejects the request.
+       It receives the source string (which contains protocol, host, and port parts) as parameter and
+       return ``True/False`` to indicate that the server accepts/rejects the request.
        If ``check_origin`` is set, the ``allowed_origins`` parameter will be ignored.
     :param bool auto_open_webbrowser: Whether or not auto open web browser when server is started (if the operating system allows it) .
     :param int/str max_payload_size: Max size of a websocket message which Tornado can accept.
@@ -356,8 +346,13 @@ def start_server(applications, port=0, host='',
 
     cdn = cdn_validation(cdn, 'warn')  # if CDN is not available, warn user and disable CDN
 
-    utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+    page.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
 
+    debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
+
+    # Since some cloud server may close idle connections (such as heroku),
+    # use `websocket_ping_interval` to  keep the connection alive
+    tornado_app_settings.setdefault('websocket_ping_interval', 30)
     tornado_app_settings.setdefault('websocket_max_message_size', max_payload_size)  # Backward compatible
     tornado_app_settings['websocket_max_message_size'] = parse_file_size(
         tornado_app_settings['websocket_max_message_size'])
@@ -367,14 +362,13 @@ def start_server(applications, port=0, host='',
     _, port = _setup_server(webio_handler=handler, port=port, host=host, static_dir=static_dir,
                             max_buffer_size=max_payload_size, **tornado_app_settings)
 
-    print('Listen on %s:%s' % (host or '0.0.0.0', port))
+    print_listen_address(host, port)
 
     if auto_open_webbrowser:
         tornado.ioloop.IOLoop.current().spawn_callback(open_webbrowser_on_server_started, host or 'localhost', port)
 
-    if remote_access or remote_access == {}:
-        if remote_access is True: remote_access = {}
-        start_remote_access_service(**remote_access, local_port=port)
+    if remote_access:
+        start_remote_access_service(local_port=port)
 
     tornado.ioloop.IOLoop.current().start()
 
@@ -444,7 +438,8 @@ def start_server_in_current_thread_session():
         server.stop()
         logger.debug('Closing tornado ioloop...')
         tasks = [t for t in asyncio.all_tasks() if t is not asyncio.current_task() and not t.done()]
-        for task in tasks: task.cancel()
+        for task in tasks:
+            task.cancel()
 
         # 必须需要 await asyncio.sleep ，否则上方 task.cancel() 调用无法调度生效
         # This line must be required, otherwise the `task.cancel()` call cannot be scheduled to take effect
@@ -468,7 +463,7 @@ def start_server_in_current_thread_session():
             port = int(os.environ.get("PYWEBIO_SCRIPT_MODE_PORT"))
 
         server, port = _setup_server(webio_handler=SingleSessionWSHandler, port=port, host='localhost',
-                                     websocket_max_message_size=parse_file_size('4G'))
+                                     websocket_max_message_size=parse_file_size('200M'))
         tornado.ioloop.IOLoop.current().spawn_callback(partial(wait_to_stop_loop, server=server))
 
         if "PYWEBIO_SCRIPT_MODE_PORT" not in os.environ:
