@@ -91,12 +91,18 @@ class ThreadBasedSession(Session):
                 for t in self.threads:
                     if t.is_alive() and t is not threading.current_thread():
                         t.join()
+
                 try:
-                    self.send_task_command(dict(command='close_session'))
-                except SessionClosedException:
+                    if self.need_keep_alive():
+                        from ..session import hold
+                        hold()
+                    else:
+                        self.send_task_command(dict(command='close_session'))
+                except SessionException:  # ignore SessionException error
                     pass
-                self._trigger_close_event()
-                self.close()
+                finally:
+                    self._trigger_close_event()
+                    self.close()
 
         thread = threading.Thread(target=main_task, kwargs=dict(target=target),
                                   daemon=True, name='main_task')
@@ -177,22 +183,25 @@ class ThreadBasedSession(Session):
             # so the `get_current_session()` call in those thread will raise SessionNotFoundException
             del cls.thread2session[id(t)]
 
-        if self.callback_mq is not None:  # 回调功能已经激活, 结束回调线程
-            mq = queue.Queue(maxsize=1)
-            mq.put(None)
-            self.callback_mq = mq
+        if self.callback_thread:
+            del cls.thread2session[id(self.callback_thread)]
 
-        for mq in self.task_mqs.values():
-            for _ in range(2):
+        def try_best_to_add_item_to_mq(mq, item, try_count=10):
+            for _ in range(try_count):
                 try:
-                    mq.put(None, block=not nonblock)  # 消费端接收到None消息会抛出SessionClosedException异常
-                    break
+                    mq.put(item, block=False)
+                    return True
                 except queue.Full:
                     try:
                         mq.get(block=False)
                     except queue.Empty:
                         pass
 
+        if self.callback_mq is not None:  # 回调功能已经激活, 结束回调线程
+            try_best_to_add_item_to_mq(self.callback_mq, None)
+
+        for mq in self.task_mqs.values():
+            try_best_to_add_item_to_mq(mq, None)  # 消费端接收到None消息会抛出SessionClosedException异常
         self.task_mqs = {}
 
     def close(self, nonblock=False):
@@ -246,6 +255,7 @@ class ThreadBasedSession(Session):
                 except Exception:
                     # 子类可能会重写 get_current_session ，所以不要用 ThreadBasedSession.get_current_session 来调用
                     self.get_current_session().on_task_exception()
+                # todo: clean up from `register_thread()`
 
             if mutex:
                 run(callback)
@@ -281,6 +291,10 @@ class ThreadBasedSession(Session):
         event_mq = queue.Queue(maxsize=self.event_mq_maxsize)  # 线程内的用户事件队列
         self.task_mqs[self._get_task_id(t)] = event_mq
 
+    def need_keep_alive(self) -> bool:
+        # if callback thread is activated, then the session need to keep alive
+        return self.callback_thread is not None
+
 
 class ScriptModeSession(ThreadBasedSession):
     """Script mode的会话实现"""
